﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Puzzle
{
    public partial class Form1 : Form
    {
        private int ButtonsX = 4;   // 퍼즐의 가로 갯수를 지정 (4는 임시의 값)
        private int ButtonsY = 4;   // 퍼즐의 세로 갯수를 지정 (4는 임시의 값)
        private static Button[,] btn;   // 버튼 생성 및 이용을 위해 배열을 이용해 선언
        private int ButtonXSize, ButtonYSize;   // 퍼즐용 버튼의 가로, 세로 크기
        private int blankX, blankY; // 빈칸의 X,Y의 좌표
        private int remainTime; // 남은 시간
        private bool gameStart; // 게임 시작 체크
        
        public Form1()
        {
            InitializeComponent();
        }

        // 퍼즐의 가로, 세로 갯수 지정
        public void setButtons(int Buttons)
        {
            this.ButtonsX = Buttons;    // 입력받은 값으로 퍼즐 가로갯수 지정
            this.ButtonsY = Buttons;    // 입력받은 값으로 퍼즐 세로갯수 지정

            this.Size = new Size((ButtonsX * 100), (ButtonsY * 100) + 90);  // 기본 버튼 크기를 100으로 폼 사이즈 재조정
            this.Text = "숫자 퍼즐 맞추기 게임"; // 폼의 제목 지정
            createButton(); // 버튼 생성

            init(); // 게임 초기화
            this.Invalidate();  // paint 이벤트를 발생시키기 위해 사용
        }

        // 게임 초기화
        private void init()
        {
            gameStart = false;  // 게임 시작 정지 상태 설정

            remainTime = 6000;  // 남은 시간 6000으로 설정
            label1.Text = "남은 시간 : " + remainTime.ToString();   // 남은 시간 레이블에 표시
            
            randomPuzzle();     // 퍼즐 섞는 구문 실행

            timer1.Enabled = true;  // 타이머 진행
            gameStart = true;   // 게임 시작 상태 설정
        }

        // 초기에 처음 퍼즐용 버튼을 생성
        private void createButton()
        {
            btn = new Button[ButtonsX, ButtonsY]; // 퍼즐용 버튼 생성을 위한 선언
            int Number = 1; // 버튼에 번호를 순서대로 기록하기 위해 변수 선언

            for (int index2 = 0; index2 < ButtonsY; index2++)   // Y좌표를 설정하기 위해 순서대로 진행
            {
                for (int index = 0; index < ButtonsX; index++)  // X좌표를 설정하기 위해 순서대로 진행
                {
                    btn[index, index2] = new Button();  // 좌표를 이용하기 위해 배열의 형태로 새로운 버튼 생성
                    btn[index, index2].Click += new EventHandler(OnClick);  // 버튼의 클릭 이벤트를 설정

                    btn[index, index2].Name = Number.ToString() + "Button"; // 버튼의 이름을 순서대로 설정
                    btn[index, index2].Visible = true;  // 기본적으로 버튼이 보이도록 설정
                    Number++;   // 버튼의 이름으로 이용하는 숫자 하나 증가
                    this.Controls.Add(btn[index, index2]);  // 버튼을 폼에 적용
                }
            }
        }

        // 폼의 크기 변경과 같이 버튼을 다시 그릴 필요가 있을 때
        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            for (int index2 = 0; index2 < ButtonsY; index2++)   // Y좌표의 버튼을 순서대로 진행
            {
                for (int index = 0; index < ButtonsX; index++)  // X좌표의 버튼을 순서대로 진행
                {
                    btn[index, index2].Size = new Size(this.ButtonXSize, this.ButtonYSize); // 버튼의 크기를 이미 설정한 버튼 하나의 크기로 설정
                    btn[index, index2].Location = new Point((index * this.ButtonXSize), (index2 * this.ButtonYSize)+25);
                        // 버튼의 위치를 좌표와 버튼의 크기를 이용하여 버튼의 새 위치를 설정
                }
            }
        }

        // 폼 변경에 따른 버튼및 레이블의 디자인 변경
        private void Form1_Resize(object sender, EventArgs e)
        {
            int width = this.ClientSize.Width;  // 폼의 넓이를 받음
            int height = this.ClientSize.Height;    // 폼의 높이를 받음

            this.ButtonXSize = width / ButtonsX;    // 넓이를 버튼의 수로 나누어 버튼 하나의 넓이를 구함
            this.ButtonYSize = (height - 60) / ButtonsY;    // 높이중 레이블과 메뉴만큼을 제외한 크기를 버튼의 수로 나누어 버튼 하나의 높이를 구함

            label1.Location = new Point((width-200),(height-30));   // 남은시간을 표시할 레이블을 새로운 폼 크기에 맞추어 새로 지정

            this.Invalidate();  // Paint 이벤트를 발생시키기 위해 사용
        }

        // 빈칸과 숫자 칸 변경
        private void change(int index, int index2)
        {
            btn[blankX,blankY].Text = btn[index,index2].Text;   // 입력받은 위치의 숫자를 빈칸에 입력
            
            btn[blankX,blankY].Visible = true;  // 빈칸을 보이도록 설정
            btn[blankX, blankY].Focus();    // 빈칸에 포커스 맞추기
            btn[index, index2].Text = Convert.ToString(ButtonsX * ButtonsY);    // 입력받은 위치에 마지막 숫자 입력
            btn[index,index2].Visible = false;  // 입력받은 위치를 보이지 않도록 설정

            blankX = index; // 빈칸의 X좌표를 입력받은 위치의 X좌표로 변경
            blankY = index2;    // 빈칸의 Y좌표를 입력받은 위치의 Y좌표로 변경
        }

        /* 빈칸의 바로 옆칸만 이동 가능한 이벤트
        private void OnClick(object sender, EventArgs e)
        {
            for (int index = 0; index < ButtonsX; index++)  // X 좌표를 순서대로 진행
            {
                for (int index2 = 0; index2 < ButtonsY; index2++)   // Y 좌표를 순서대로 진행
                {
                    if ((Button) sender == btn[index,index2])   // 클릭한 버튼을 확인
                    {
                        if (index == (blankX - 1) || index == (blankX + 1)) // 빈칸의 좌,우에 위치하는 버튼인지 확인 
                        {
                            if (index2 == blankY)   // Y좌표가 같은지 확인
                            {   
                                change(index, index2);  // 선택한 버튼과 빈칸 내용 변경
                            }
                        }
                        else if (index2 == (blankY - 1) || index2 == (blankY + 1))  // 빈칸의 위,아래에 위치하는 버튼인지 확인하여 위와 같이 진행
                        {
                            if (index == blankX)
                            {
                                change(index, index2);
                            }
                        }
                    }
                }
            }
        }
        */

        // 여러칸 함께 이동이 가능한 이벤트 
        private void OnClick(object sender, EventArgs e)
        {

            if (gameStart)  // 게임 시작 상태 확인
            {
                for (int index = 0; index < ButtonsX; index++)  // X 좌표를 순서대로 진행
                {
                    for (int index2 = 0; index2 < ButtonsY; index2++)   // Y 좌표를 순서대로 진행
                    {
                        if ((Button)sender == btn[index, index2])   // 클릭한 버튼을 확인
                        {
                            if (index == blankX)    // 빈칸과 같은 X좌표인지 확인
                            {
                                if (index2 > blankY)    // Y좌표가 빈칸보다 아래쪽인지 확인
                                {
                                    for (int count = (blankY + 1); count <= index2; count++)
                                        change(index, count);   // 빈칸에서 선택한 버튼까지 차래대로 빈칸과 위치 변경
                                }
                                else if (index2 < blankY)   // Y좌표가 빈칸보다 위쪽인지 확인
                                {
                                    for (int count = (blankY - 1); count >= index2; count--)
                                        change(index, count);   // 빈칸에서 선택한 버튼까지 차래대로 빈칸과 숫자 변경
                                }
                            }
                            else if (index2 == blankY)  // 빈칸과 같은 Y좌표인지 확인하여 위와 같이 진행
                            {
                                if (index > blankX)
                                {
                                    for (int count = (blankX + 1); count <= index; count++)
                                        change(count, index2);
                                }
                                else if (index < blankX)
                                {
                                    for (int count = (blankX - 1); count >= index; count--)
                                        change(count, index2);
                                }
                            }
                        }
                    }
                }
                completeCheck();    // 완료상태 확인
            }
        }

        // 퍼즐 섞는 구문
        private void randomPuzzle()
        {
            Random random = new Random(); // 난수를 이용하기 위해 선언

            int[] compareNumber = new int[(ButtonsX*ButtonsY)]; // 중복을 피하기 위해 변수 선언
            int tempNumber; // 난수를 발생한 숫자를 임시로 기억하기 위한 변수

            for (int count = 1; count <= (ButtonsX*ButtonsY); count++)  
                compareNumber[count - 1] = count;   // 중복을 피하기 위한 변수에 순서대로 버튼의 수만큼의 숫자를 입력

            for (int count = 0; count < (ButtonsX * ButtonsY); count++) // 버튼의 처음부터 마지막까지 순서대로 진행
            {
                do
                {
                    tempNumber = random.Next(1, (ButtonsX * ButtonsY)+1);   // 난수를 이용하여 숫자 하나 선택
                } while (compareNumber[tempNumber-1] == 99);    // 중복일 경우 숫자 선택 반복

                compareNumber[tempNumber-1] = 99;   // 선택된 숫자의 경우 99로 선택된 숫자임을 지정

                btn[(count % ButtonsX), (count / ButtonsY)].Text = tempNumber.ToString();   // 처음부터 차래대로 선택된 수 표시
                if (tempNumber == (ButtonsX * ButtonsY))    // 선택된 숫자가 마지막일 경우
                {
                    btn[(count % ButtonsX), (count / ButtonsY)].Visible = false;    // 보이지 않도록 설정
                    blankX = (count % ButtonsX);    // 빈칸의 X좌표 기록
                    blankY = (count / ButtonsY);    // 빈칸의 Y좌표 기록
                }
                else btn[(count % ButtonsX), (count / ButtonsY)].Visible = true;    // 일반 버튼일 경우 보이도록 설정

            }
        }

        // 게임 완료 체크
        private void completeCheck()
        {
            Boolean check = true; // 완료했는지 확인기 위한 변수

            int number, number2;    // 한 버튼의 숫자와 다음 버튼의 숫자를 받기 위한 변수

            for (int index = 0; index < (ButtonsX * ButtonsY)-1; index++)   // 총 버튼의 수 만큼 진행(-1은 빈칸 다음 버튼이 없기 때문)
            {
                number = Convert.ToInt32(btn[(index % ButtonsX), (index / ButtonsY)].Text); // 처음부터 진행하며 버튼의 숫자를 받아옴
                number2 = Convert.ToInt32(btn[((index + 1) % ButtonsX), ((index + 1) / ButtonsY)].Text);    // 바로 다음 버튼의 숫자를 받아옴

                if (number > number2)   // 첫번째 숫자가 두번째 숫자보다 큰지를 확인
                {
                    check = false;  // 완료 상태가 아님을 기록
                    break;  // 더이상 확인할 필요 없으므로 중단
                }
            }

            if (check)  // 완료 했는지 확인
            {
                timer1.Enabled = false; // 진행을 중지하기 위해 타이머 정지
                MessageBox.Show("축하합니다!! 퍼즐을 완료하셨습니다~!!", "게임 완료!"); // 게임 완료 표시를 위해 메시지 박스 표시
                init(); // 게임 초기화
            }
        }

        // 시간의 흐름에 따른 남은 시간 표시 및 게임의 종료 체크
        private void timer1_Tick(object sender, EventArgs e)
        {
            remainTime--;   // 남은시간 1 감소
            label1.Text = "남은 시간 : " + remainTime.ToString();   // 레이블애 남은시간 표시
            if (remainTime == 0)    // 남은 시간이 0일 경우
            {
                timer1.Enabled = false; // 진행을 멈추기 위해 타이머 정지
                MessageBox.Show("제한 시간이 종료되었습니다!!", "게임 종료");   // 게임 종료 메시지 표시
                init();     // 게임 초기화
            }
        }

        // 상단 메뉴 섞기 버튼
        private void 섞기SToolStripMenuItem_Click(object sender, EventArgs e)
        {
            randomPuzzle(); // 섞기 진행
            init(); // 게임 초기화
        }

        private void 끝내기XToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();   // 게임 종료
        }

        // 게임의 시작시에 가로 세로 퍼즐의 갯수를 입력받기 위한 창 나타내기
        private void Form1_Load(object sender, EventArgs e)
        {
            puzzleForm pf = new puzzleForm(this);   // 입력을 위한 창 선언(입력받은 갯수를 넘겨받기 위해 자신을 보냄)
            pf.ShowDialog();    // 모달 형식으로 창 보이기
        }
    }
}